@@ -1,15 +1,15 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
-#ifndef __AFSDICON_H_ENV__
-#define __AFSDICON_H_ENV__ 1
+#ifndef OPENAFS_WINNT_AFSD_AFSDICON_H
+#define OPENAFS_WINNT_AFSD_AFSDICON_H 1
 
 #define IDM_HELP 100
 
-#endif /* __AFSDICON_H_ENV */
+#endif /* OPENAFS_WINNT_AFSD_AFSDICON_H */
