@@ -279,6 +279,14 @@ cm_ResetServerBusyStatus(cm_serverRef_t **serverspp)
     lock_ReleaseRead(&cm_serverLock);
 }
 
+static int
+isNetworkError(int code)
+{
+    if (code >= -64 && code < 0)
+	return 1;
+    return 0;
+}
+
 /*
  * Analyze the error return from an RPC.  Determine whether or not to retry,
  * and if we're going to retry, determine whether failover is appropriate,
@@ -971,27 +979,6 @@ cm_Analyze(cm_conn_t *connp,
 
         retry = 2;
     }
-    else if (errorCode == RX_CALL_BUSY) {
-        /*
-         * RPC failed because the selected call channel
-         * is currently busy on the server.  Unconditionally
-         * retry the request so an alternate call channel can be used.
-         */
-	if (connp)
-	    _InterlockedAnd(&connp->flags, ~CM_CONN_FLAG_NEW);
-
-        if (serverp)
-            sprintf(addr, "%d.%d.%d.%d",
-                    ((serverp->addr.sin_addr.s_addr & 0xff)),
-                    ((serverp->addr.sin_addr.s_addr & 0xff00)>> 8),
-                    ((serverp->addr.sin_addr.s_addr & 0xff0000)>> 16),
-                    ((serverp->addr.sin_addr.s_addr & 0xff000000)>> 24));
-
-        LogEvent(EVENTLOG_WARNING_TYPE, MSG_RX_BUSY_CALL_CHANNEL, addr);
-        osi_Log1(afsd_logp, "cm_Analyze: Retry RPC due to busy call channel addr[%s]",
-                 osi_LogSaveString(afsd_logp,addr));
-        retry = 2;
-    }
     else if (errorCode == VNOSERVICE) {
         /*
          * The server did not service the RPC.
@@ -1022,85 +1009,6 @@ cm_Analyze(cm_conn_t *connp,
 
 	retry = 2;
     }
-    else if (errorCode == RX_CALL_IDLE) {
-        /*
-         * RPC failed because the server failed to respond with data
-         * within the idle dead timeout period.  This could be for a variety
-         * of reasons:
-         *  1. The server could have a bad partition such as a failed
-         *     disk or iSCSI target and all I/O to that partition is
-         *     blocking on the server and will never complete.
-         *
-         *  2. The server vnode may be locked by another client request
-         *     that is taking a very long time.
-         *
-         *  3. The server may have a very long queue of requests
-         *     pending and is unable to process this request.
-         *
-         *  4. The server could be malicious and is performing a denial
-         *     of service attack against the client.
-         *
-         * If this is a request against a .readonly with alternate sites
-         * the server should be marked down for this request and the
-         * client should fail over to another server.  If this is a
-         * request against a single source, the client may retry once.
-         */
-	if (connp)
-	    _InterlockedAnd(&connp->flags, ~CM_CONN_FLAG_NEW);
-
-        if (serverp)
-            sprintf(addr, "%d.%d.%d.%d",
-                    ((serverp->addr.sin_addr.s_addr & 0xff)),
-                    ((serverp->addr.sin_addr.s_addr & 0xff00)>> 8),
-                    ((serverp->addr.sin_addr.s_addr & 0xff0000)>> 16),
-                    ((serverp->addr.sin_addr.s_addr & 0xff000000)>> 24));
-
-        if (fidp) {
-            code = cm_FindVolumeByID(cellp, fidp->volume, userp, reqp,
-                                     CM_GETVOL_FLAG_NO_LRU_UPDATE,
-                                     &volp);
-            if (code == 0) {
-                statep = cm_VolumeStateByID(volp, fidp->volume);
-
-                if (statep)
-                    replicated = (statep->flags & CM_VOL_STATE_FLAG_REPLICATED);
-
-                lock_ObtainRead(&cm_volumeLock);
-                cm_PutVolume(volp);
-                lock_ReleaseRead(&cm_volumeLock);
-                volp = NULL;
-            }
-
-            if (storeOp)
-                scp = cm_FindSCache(fidp);
-	    if (scp) {
-                if (cm_HaveCallback(scp)) {
-                    lock_ObtainWrite(&scp->rw);
-                    cm_DiscardSCache(scp);
-                    lock_ReleaseWrite(&scp->rw);
-
-                    /*
-                     * We really should notify the redirector that we discarded
-                     * the status information but doing so in this case is not
-                     * safe as it can result in a deadlock with extent release
-                     * processing.
-                     */
-                }
-                cm_ReleaseSCache(scp);
-            }
-        }
-
-        if (replicated && serverp) {
-            reqp->errorServp = serverp;
-            reqp->tokenError = errorCode;
-
-	    retry = 2;
-        }
-
-        LogEvent(EVENTLOG_WARNING_TYPE, MSG_RX_IDLE_DEAD_TIMEOUT, addr, retry);
-        osi_Log2(afsd_logp, "cm_Analyze: RPC failed due to idle dead timeout addr[%s] retry=%u",
-                 osi_LogSaveString(afsd_logp,addr), retry);
-    }
     else if (errorCode == RX_CALL_DEAD) {
         /* mark server as down */
         if (serverp)
@@ -1151,9 +1059,23 @@ cm_Analyze(cm_conn_t *connp,
         if ( timeLeft > 2 )
             retry = 1;
     }
-    else if (errorCode >= -64 && errorCode < 0) {
-        /* mark server as down */
-        if (serverp)
+    else if (isNetworkError(errorCode)) {
+	/*
+	 * any other rx error - mark server as down.
+	 *
+	 * The most likely error to receive is RX_INVALID_OPERATION which
+	 * can be received as a result of an unrecognized rx service id or
+	 * an unrecognized security class.  Failing to mark the server down
+	 * when a service id or security class that is expected to be
+	 * recognized is not will result in an infinite retry loop.  Instead
+	 * mark the server down.  If we are ever able to successfully
+	 * communicate with it again it will be marked up via a probe.
+	 *
+	 * The most common scenario for this behavior would be an AuriStor
+	 * file server being replaced by an AFS3 file server or a reflected
+	 * packet.
+	 */
+	if (serverp)
             sprintf(addr, "%d.%d.%d.%d",
                     ((serverp->addr.sin_addr.s_addr & 0xff)),
                     ((serverp->addr.sin_addr.s_addr & 0xff00)>> 8),
@@ -1168,9 +1090,13 @@ cm_Analyze(cm_conn_t *connp,
         if (serverp) {
 	    if ((connp->flags & CM_CONN_FLAG_NEW) ||
 		(reqp->flags & CM_REQ_NEW_CONN_FORCED)) {
-                reqp->errorServp = serverp;
-                reqp->tokenError = errorCode;
-            } else {
+		lock_ObtainMutex(&serverp->mx);
+		if (!(serverp->flags & CM_SERVERFLAG_DOWN)) {
+		    _InterlockedOr(&serverp->flags, CM_SERVERFLAG_DOWN);
+		    serverp->downTime = time(NULL);
+		}
+		lock_ReleaseMutex(&serverp->mx);
+	    } else {
                 reqp->flags |= CM_REQ_NEW_CONN_FORCED;
                 forcing_new = 1;
                 cm_ForceNewConnections(serverp);
@@ -1377,10 +1303,10 @@ cm_Analyze(cm_conn_t *connp,
     }
 
     /* If not allowed to retry, don't */
-    if (dead_session ||
-	!forcing_new && (retry < 2) && (reqp->flags & CM_REQ_NORETRY) &&
-        !(errorCode > -64 && errorCode <= RX_INVALID_OPERATION))
-        retry = 0;
+    if (dead_session)
+	retry = 0;
+    else if (!forcing_new && (retry < 2) && (reqp->flags & CM_REQ_NORETRY))
+	retry = 0;
 
     /* drop this on the way out */
     if (connp)
@@ -1610,7 +1536,8 @@ static void cm_NewRXConnection(cm_conn_t *tcp, cm_ucell_t *ucellp,
         rx_SetConnHardDeadTime(tcp->rxconnp, HardDeadtimeout);
 
     /*
-     * Setting idle dead timeout to a non-zero value activates RX_CALL_IDLE errors
+     * Setting idle dead timeout to a non-zero value activates RX_CALL_TIMEOUT
+     * errors if the call is idle for a certain amount of time.
      */
     if (replicated) {
 	_InterlockedOr(&tcp->flags, CM_CONN_FLAG_REPLICATION);
