@@ -22,12 +22,14 @@
  * SOFTWARE.
  */
 
+#include <afsconfig.h>
+#include <afs/param.h>
+#include <afs/stds.h>
+#include <roken.h>
+
 #include <windows.h>
-#include <stdlib.h>
 #include <wchar.h>
 #include <strsafe.h>
-#include <stdio.h>
-#include <errno.h>
 
 #include "cm_nls.h"
 
@@ -146,7 +148,7 @@ long cm_InitNormalization(void)
    buffer is different from the supplied buffer and non-NULL, it
    should be freed using free().
 */
-static wchar_t * 
+static wchar_t *
 NormalizeUtf16String(const wchar_t * src, int cch_src, wchar_t * ext_dest, int *pcch_dest)
 {
     if (!nls_init)
@@ -520,7 +522,7 @@ static int sanitize_bytestring(const char * src, int cch_src,
 
         rc = sanitized_escapes_1252[*src];
         if (IS_ESCAPED(rc)) {
-            static const char hex[] = 
+            static const char hex[] =
                 {'0','1','2','3','4','5','6','7',
                  '8','9','a','b','c','d','e','f'};
 
@@ -1113,6 +1115,9 @@ int cm_strnicmp_utf8(const char * str1, const char * str2, int n)
     if (!nls_init)
         cm_InitNormalization();
 
+    if (n == 0)
+        return 0;
+
     /* first check for NULL pointers (assume NULL < "") */
     if (str1 == NULL) {
         if (str2 == NULL)
@@ -1159,6 +1164,9 @@ int cm_strnicmp_utf16(const cm_unichar_t * str1, const cm_unichar_t * str2, int
     if (!nls_init)
         cm_InitNormalization();
 
+    if (len == 0)
+        return 0;
+
     /* first check for NULL pointers */
     if (str1 == NULL) {
         if (str2 == NULL)
@@ -1301,23 +1309,6 @@ int cm_stricmp_utf8(const char * str1, const char * str2)
     }
 }
 
-#if 0
-wchar_t * strupr_utf16(wchar_t * wstr, size_t cbstr)
-{
-    wchar_t wstrd[NLSMAXCCH];
-    int len;
-
-    if (!nls_init)
-        cm_InitNormalization();
-
-    len = cbstr / sizeof(wchar_t);
-    len = LCMapStringW(nls_lcid, LCMAP_UPPERCASE, wstr, len, wstrd, NLSMAXCCH);
-    StringCbCopyW(wstr, cbstr, wstrd);
-
-    return wstr;
-}
-#endif
-
 char * strupr_utf8(char * str, size_t cbstr)
 {
     wchar_t wstr[NLSMAXCCH];
