@@ -48,6 +48,14 @@ AFSFastIoCheckIfPossible( IN struct _FILE_OBJECT *FileObject,
                           OUT PIO_STATUS_BLOCK IoStatus,
                           IN struct _DEVICE_OBJECT *DeviceObject)
 {
+    UNREFERENCED_PARAMETER(FileObject);
+    UNREFERENCED_PARAMETER(FileOffset);
+    UNREFERENCED_PARAMETER(Length);
+    UNREFERENCED_PARAMETER(Wait);
+    UNREFERENCED_PARAMETER(LockKey);
+    UNREFERENCED_PARAMETER(CheckForReadOperation);
+    UNREFERENCED_PARAMETER(IoStatus);
+    UNREFERENCED_PARAMETER(DeviceObject);
 
     BOOLEAN bStatus = FALSE;
 
@@ -65,6 +73,15 @@ AFSFastIoRead( IN struct _FILE_OBJECT *FileObject,
                IN struct _DEVICE_OBJECT *DeviceObject)
 {
 
+    UNREFERENCED_PARAMETER(FileObject);
+    UNREFERENCED_PARAMETER(FileOffset);
+    UNREFERENCED_PARAMETER(Length);
+    UNREFERENCED_PARAMETER(Wait);
+    UNREFERENCED_PARAMETER(LockKey);
+    UNREFERENCED_PARAMETER(Buffer);
+    UNREFERENCED_PARAMETER(IoStatus);
+    UNREFERENCED_PARAMETER(DeviceObject);
+
     BOOLEAN bStatus = FALSE;
 
     return bStatus;
@@ -81,6 +98,15 @@ AFSFastIoWrite( IN struct _FILE_OBJECT *FileObject,
                 IN struct _DEVICE_OBJECT *DeviceObject)
 {
 
+    UNREFERENCED_PARAMETER(FileObject);
+    UNREFERENCED_PARAMETER(FileOffset);
+    UNREFERENCED_PARAMETER(Length);
+    UNREFERENCED_PARAMETER(Wait);
+    UNREFERENCED_PARAMETER(LockKey);
+    UNREFERENCED_PARAMETER(Buffer);
+    UNREFERENCED_PARAMETER(IoStatus);
+    UNREFERENCED_PARAMETER(DeviceObject);
+
     BOOLEAN bStatus = FALSE;
 
     return bStatus;
@@ -93,6 +119,11 @@ AFSFastIoQueryBasicInfo( IN struct _FILE_OBJECT *FileObject,
                          OUT PIO_STATUS_BLOCK IoStatus,
                          IN struct _DEVICE_OBJECT *DeviceObject)
 {
+    UNREFERENCED_PARAMETER(FileObject);
+    UNREFERENCED_PARAMETER(Wait);
+    UNREFERENCED_PARAMETER(Buffer);
+    UNREFERENCED_PARAMETER(IoStatus);
+    UNREFERENCED_PARAMETER(DeviceObject);
 
     BOOLEAN bStatus = FALSE;
 
@@ -106,6 +137,11 @@ AFSFastIoQueryStandardInfo( IN struct _FILE_OBJECT *FileObject,
                             OUT PIO_STATUS_BLOCK IoStatus,
                             IN struct _DEVICE_OBJECT *DeviceObject)
 {
+    UNREFERENCED_PARAMETER(FileObject);
+    UNREFERENCED_PARAMETER(Wait);
+    UNREFERENCED_PARAMETER(Buffer);
+    UNREFERENCED_PARAMETER(IoStatus);
+    UNREFERENCED_PARAMETER(DeviceObject);
 
     BOOLEAN bStatus = FALSE;
 
@@ -123,6 +159,15 @@ AFSFastIoLock( IN struct _FILE_OBJECT *FileObject,
                OUT PIO_STATUS_BLOCK IoStatus,
                IN struct _DEVICE_OBJECT *DeviceObject)
 {
+    UNREFERENCED_PARAMETER(FileObject);
+    UNREFERENCED_PARAMETER(FileOffset);
+    UNREFERENCED_PARAMETER(Length);
+    UNREFERENCED_PARAMETER(ProcessId);
+    UNREFERENCED_PARAMETER(Key);
+    UNREFERENCED_PARAMETER(FailImmediately);
+    UNREFERENCED_PARAMETER(ExclusiveLock);
+    UNREFERENCED_PARAMETER(IoStatus);
+    UNREFERENCED_PARAMETER(DeviceObject);
 
     BOOLEAN bStatus = FALSE;
 
@@ -138,6 +183,13 @@ AFSFastIoUnlockSingle( IN struct _FILE_OBJECT *FileObject,
                        OUT PIO_STATUS_BLOCK IoStatus,
                        IN struct _DEVICE_OBJECT *DeviceObject)
 {
+    UNREFERENCED_PARAMETER(FileObject);
+    UNREFERENCED_PARAMETER(FileOffset);
+    UNREFERENCED_PARAMETER(Length);
+    UNREFERENCED_PARAMETER(ProcessId);
+    UNREFERENCED_PARAMETER(Key);
+    UNREFERENCED_PARAMETER(IoStatus);
+    UNREFERENCED_PARAMETER(DeviceObject);
 
     BOOLEAN bStatus = FALSE;
 
@@ -150,6 +202,10 @@ AFSFastIoUnlockAll( IN struct _FILE_OBJECT *FileObject,
                     OUT PIO_STATUS_BLOCK IoStatus,
                     IN struct _DEVICE_OBJECT *DeviceObject)
 {
+    UNREFERENCED_PARAMETER(FileObject);
+    UNREFERENCED_PARAMETER(ProcessId);
+    UNREFERENCED_PARAMETER(IoStatus);
+    UNREFERENCED_PARAMETER(DeviceObject);
 
     BOOLEAN bStatus = FALSE;
 
@@ -163,6 +219,11 @@ AFSFastIoUnlockAllByKey( IN struct _FILE_OBJECT *FileObject,
                          OUT PIO_STATUS_BLOCK IoStatus,
                          IN struct _DEVICE_OBJECT *DeviceObject)
 {
+    UNREFERENCED_PARAMETER(FileObject);
+    UNREFERENCED_PARAMETER(ProcessId);
+    UNREFERENCED_PARAMETER(Key);
+    UNREFERENCED_PARAMETER(IoStatus);
+    UNREFERENCED_PARAMETER(DeviceObject);
 
     BOOLEAN bStatus = FALSE;
 
@@ -180,6 +241,15 @@ AFSFastIoDevCtrl( IN struct _FILE_OBJECT *FileObject,
                   OUT PIO_STATUS_BLOCK IoStatus,
                   IN struct _DEVICE_OBJECT *DeviceObject)
 {
+    UNREFERENCED_PARAMETER(FileObject);
+    UNREFERENCED_PARAMETER(Wait);
+    UNREFERENCED_PARAMETER(InputBuffer);
+    UNREFERENCED_PARAMETER(InputBufferLength);
+    UNREFERENCED_PARAMETER(OutputBuffer);
+    UNREFERENCED_PARAMETER(OutputBufferLength);
+    UNREFERENCED_PARAMETER(IoControlCode);
+    UNREFERENCED_PARAMETER(IoStatus);
+    UNREFERENCED_PARAMETER(DeviceObject);
 
     BOOLEAN bStatus = FALSE;
 
@@ -192,15 +262,37 @@ AFSFastIoAcquireFile( IN struct _FILE_OBJECT *FileObject)
 
     AFSFcb *pFcb = (AFSFcb *)FileObject->FsContext;
 
-    AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+    AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                   AFS_TRACE_LEVEL_VERBOSE,
-                  "AFSFastIoAcquireFile Acquiring Fcb SectionObject lock %08lX EXCL %08lX\n",
+		  "AFSFastIoAcquireFile Acquiring Fcb lock %p EXCL %08lX\n",
+		  &pFcb->NPFcb->Resource,
+		  PsGetCurrentThread()));
+
+    AFSAcquireExcl( &pFcb->NPFcb->Resource,
+		    TRUE);
+
+    AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
+		  AFS_TRACE_LEVEL_VERBOSE,
+                  "AFSFastIoAcquireFile Acquiring Fcb SectionObject lock %p EXCL %08lX\n",
                   &pFcb->NPFcb->SectionObjectResource,
-                  PsGetCurrentThread());
+                  PsGetCurrentThread()));
 
     AFSAcquireExcl( &pFcb->NPFcb->SectionObjectResource,
                     TRUE);
 
+    if( NULL == pFcb->Specific.File.SectionCreateFO )
+    {
+	//
+	// If not re-entrant then save and reference
+	//
+	pFcb->Specific.File.SectionCreateFO =
+	  CcGetFileObjectFromSectionPtrs( &pFcb->NPFcb->SectionObjectPointers );
+	if( NULL !=  pFcb->Specific.File.SectionCreateFO )
+	{
+	    ObReferenceObject( pFcb->Specific.File.SectionCreateFO);
+	}
+    }
+
     return;
 }
 
@@ -210,16 +302,38 @@ AFSFastIoReleaseFile( IN struct _FILE_OBJECT *FileObject)
 
     AFSFcb *pFcb = (AFSFcb *)FileObject->FsContext;
 
+    if( ExIsResourceAcquiredExclusiveLite( &pFcb->NPFcb->Resource))
+    {
+	AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
+		      AFS_TRACE_LEVEL_VERBOSE,
+		      "AFSFastIoReleaseFile Releasing Fcb Lock %p EXCL %08lX\n",
+		      &pFcb->NPFcb->Resource,
+		      PsGetCurrentThread()));
+
+	AFSReleaseResource( &pFcb->NPFcb->Resource);
+    }
+
     if( ExIsResourceAcquiredExclusiveLite( &pFcb->NPFcb->SectionObjectResource))
     {
+	PFILE_OBJECT fileObject = pFcb->Specific.File.SectionCreateFO;
+	pFcb->Specific.File.SectionCreateFO = NULL;
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSFastIoReleaseFile Releasing Fcb SectionObject lock %08lX EXCL %08lX\n",
+                      "AFSFastIoReleaseFile Releasing Fcb SectionObject lock %p EXCL %08lX\n",
                       &pFcb->NPFcb->SectionObjectResource,
-                      PsGetCurrentThread());
+                      PsGetCurrentThread()));
 
         AFSReleaseResource( &pFcb->NPFcb->SectionObjectResource);
+
+	//
+	// Now defer the Cc file object (if there was one) now that we are lock free
+	//
+
+	if( NULL != fileObject )
+	{
+	    ObDereferenceObject( fileObject );
+	}
     }
 
     return;
@@ -229,6 +343,8 @@ VOID
 AFSFastIoDetachDevice( IN struct _DEVICE_OBJECT *SourceDevice,
                        IN struct _DEVICE_OBJECT *TargetDevice)
 {
+    UNREFERENCED_PARAMETER(SourceDevice);
+    UNREFERENCED_PARAMETER(TargetDevice);
 
     return;
 }
@@ -240,6 +356,11 @@ AFSFastIoQueryNetworkOpenInfo( IN struct _FILE_OBJECT *FileObject,
                                OUT struct _IO_STATUS_BLOCK *IoStatus,
                                IN struct _DEVICE_OBJECT *DeviceObject)
 {
+    UNREFERENCED_PARAMETER(FileObject);
+    UNREFERENCED_PARAMETER(Wait);
+    UNREFERENCED_PARAMETER(Buffer);
+    UNREFERENCED_PARAMETER(IoStatus);
+    UNREFERENCED_PARAMETER(DeviceObject);
 
     BOOLEAN bStatus = FALSE;
 
@@ -255,6 +376,13 @@ AFSFastIoMdlRead( IN struct _FILE_OBJECT *FileObject,
                   OUT PIO_STATUS_BLOCK IoStatus,
                   IN struct _DEVICE_OBJECT *DeviceObject)
 {
+    UNREFERENCED_PARAMETER(FileObject);
+    UNREFERENCED_PARAMETER(FileOffset);
+    UNREFERENCED_PARAMETER(Length);
+    UNREFERENCED_PARAMETER(LockKey);
+    UNREFERENCED_PARAMETER(MdlChain);
+    UNREFERENCED_PARAMETER(IoStatus);
+    UNREFERENCED_PARAMETER(DeviceObject);
 
     BOOLEAN bStatus = FALSE;
 
@@ -266,6 +394,9 @@ AFSFastIoMdlReadComplete( IN struct _FILE_OBJECT *FileObject,
                           IN PMDL MdlChain,
                           IN struct _DEVICE_OBJECT *DeviceObject)
 {
+    UNREFERENCED_PARAMETER(FileObject);
+    UNREFERENCED_PARAMETER(MdlChain);
+    UNREFERENCED_PARAMETER(DeviceObject);
 
     BOOLEAN bStatus = FALSE;
 
@@ -281,6 +412,13 @@ AFSFastIoPrepareMdlWrite( IN struct _FILE_OBJECT *FileObject,
                           OUT PIO_STATUS_BLOCK IoStatus,
                           IN struct _DEVICE_OBJECT *DeviceObject)
 {
+    UNREFERENCED_PARAMETER(FileObject);
+    UNREFERENCED_PARAMETER(FileOffset);
+    UNREFERENCED_PARAMETER(Length);
+    UNREFERENCED_PARAMETER(LockKey);
+    UNREFERENCED_PARAMETER(MdlChain);
+    UNREFERENCED_PARAMETER(IoStatus);
+    UNREFERENCED_PARAMETER(DeviceObject);
 
     BOOLEAN bStatus = FALSE;
 
@@ -293,76 +431,30 @@ AFSFastIoMdlWriteComplete( IN struct _FILE_OBJECT *FileObject,
                            IN PMDL MdlChain,
                            IN struct _DEVICE_OBJECT *DeviceObject)
 {
+    UNREFERENCED_PARAMETER(FileObject);
+    UNREFERENCED_PARAMETER(FileOffset);
+    UNREFERENCED_PARAMETER(MdlChain);
+    UNREFERENCED_PARAMETER(DeviceObject);
 
     BOOLEAN bStatus = FALSE;
 
     return bStatus;
 }
 
-NTSTATUS
-AFSFastIoAcquireForModWrite( IN struct _FILE_OBJECT *FileObject,
-                             IN PLARGE_INTEGER EndingOffset,
-                             OUT struct _ERESOURCE **ResourceToRelease,
-                             IN struct _DEVICE_OBJECT *DeviceObject)
-{
-
-    NTSTATUS ntStatus = STATUS_FILE_LOCK_CONFLICT;
-    AFSFcb *pFcb = (AFSFcb *)FileObject->FsContext;
-
-    __Enter
-    {
-
-        if( AFSAcquireExcl( &pFcb->NPFcb->Resource,
-                            BooleanFlagOn( FileObject->Flags, FO_SYNCHRONOUS_IO)))
-        {
-
-            AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
-                          AFS_TRACE_LEVEL_VERBOSE,
-                          "AFSFastIoAcquireForModWrite Acquired Fcb SectionObject lock %08lX EXCL %08lX\n",
-                          &pFcb->NPFcb->SectionObjectResource,
-                          PsGetCurrentThread());
-
-            ntStatus = STATUS_SUCCESS;
-
-            *ResourceToRelease = &pFcb->NPFcb->SectionObjectResource;
-        }
-    }
-
-    return ntStatus;
-}
-
-NTSTATUS
-AFSFastIoReleaseForModWrite( IN struct _FILE_OBJECT *FileObject,
-                             IN struct _ERESOURCE *ResourceToRelease,
-                             IN struct _DEVICE_OBJECT *DeviceObject)
-{
-
-    NTSTATUS ntStatus = STATUS_SUCCESS;
-
-    AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
-                  AFS_TRACE_LEVEL_VERBOSE,
-                  "AFSFastIoReleaseForModWrite Releasing lock %08lX EXCL %08lX\n",
-                  ResourceToRelease,
-                  PsGetCurrentThread());
-
-    AFSReleaseResource( ResourceToRelease);
-
-    return ntStatus;
-}
-
 NTSTATUS
 AFSFastIoAcquireForCCFlush( IN struct _FILE_OBJECT *FileObject,
                             IN struct _DEVICE_OBJECT *DeviceObject)
 {
+    UNREFERENCED_PARAMETER(DeviceObject);
 
     NTSTATUS ntStatus = STATUS_FILE_LOCK_CONFLICT;
     AFSFcb *pFcb = (AFSFcb *)FileObject->FsContext;
 
-    AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+    AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                   AFS_TRACE_LEVEL_VERBOSE,
-                  "AFSFastIoAcquireForCCFlush Acquiring Fcb PagingIo lock %08lX SHARED %08lX\n",
+                  "AFSFastIoAcquireForCCFlush Acquiring Fcb PagingIo lock %p SHARED %08lX\n",
                   &pFcb->NPFcb->PagingResource,
-                  PsGetCurrentThread());
+                  PsGetCurrentThread()));
 
     AFSAcquireShared( &pFcb->NPFcb->PagingResource,
                       TRUE);
@@ -370,11 +462,11 @@ AFSFastIoAcquireForCCFlush( IN struct _FILE_OBJECT *FileObject,
     if( !ExIsResourceAcquiredSharedLite( &pFcb->NPFcb->SectionObjectResource))
     {
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSFastIoAcquireForCCFlush Acquiring Fcb SectionObject lock %08lX EXCL %08lX\n",
+                      "AFSFastIoAcquireForCCFlush Acquiring Fcb SectionObject lock %p EXCL %08lX\n",
                       &pFcb->NPFcb->SectionObjectResource,
-                      PsGetCurrentThread());
+                      PsGetCurrentThread()));
 
         AFSAcquireExcl( &pFcb->NPFcb->SectionObjectResource,
                         TRUE);
@@ -382,11 +474,11 @@ AFSFastIoAcquireForCCFlush( IN struct _FILE_OBJECT *FileObject,
     else
     {
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSFastIoAcquireForCCFlush Acquiring Fcb SectionObject lock %08lX SHARED %08lX\n",
+                      "AFSFastIoAcquireForCCFlush Acquiring Fcb SectionObject lock %p SHARED %08lX\n",
                       &pFcb->NPFcb->SectionObjectResource,
-                      PsGetCurrentThread());
+                      PsGetCurrentThread()));
 
         AFSAcquireShared( &pFcb->NPFcb->SectionObjectResource,
                           TRUE);
@@ -411,6 +503,7 @@ NTSTATUS
 AFSFastIoReleaseForCCFlush( IN struct _FILE_OBJECT *FileObject,
                             IN struct _DEVICE_OBJECT *DeviceObject)
 {
+    UNREFERENCED_PARAMETER(DeviceObject);
 
     NTSTATUS ntStatus = STATUS_SUCCESS;
     AFSFcb *pFcb = (AFSFcb *)FileObject->FsContext;
@@ -429,9 +522,9 @@ AFSFastIoReleaseForCCFlush( IN struct _FILE_OBJECT *FileObject,
     else
     {
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                       AFS_TRACE_LEVEL_ERROR,
-                      "AFSFastIoReleaseForCCFlush Called for non-acquired paging resource Fcb\n");
+                      "AFSFastIoReleaseForCCFlush Called for non-acquired paging resource Fcb\n"));
     }
 
     if( ExIsResourceAcquiredExclusiveLite( &pFcb->NPFcb->SectionObjectResource) ||
@@ -443,9 +536,9 @@ AFSFastIoReleaseForCCFlush( IN struct _FILE_OBJECT *FileObject,
     else
     {
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                       AFS_TRACE_LEVEL_ERROR,
-                      "AFSFastIoReleaseForCCFlush Called for non-acquired SectionObject resource Fcb\n");
+                      "AFSFastIoReleaseForCCFlush Called for non-acquired SectionObject resource Fcb\n"));
     }
 
     return ntStatus;
@@ -463,6 +556,16 @@ AFSFastIoReadCompressed( IN struct _FILE_OBJECT *FileObject,
                          IN ULONG CompressedDataInfoLength,
                          IN struct _DEVICE_OBJECT *DeviceObject)
 {
+    UNREFERENCED_PARAMETER(FileObject);
+    UNREFERENCED_PARAMETER(FileOffset);
+    UNREFERENCED_PARAMETER(Length);
+    UNREFERENCED_PARAMETER(LockKey);
+    UNREFERENCED_PARAMETER(Buffer);
+    UNREFERENCED_PARAMETER(MdlChain);
+    UNREFERENCED_PARAMETER(IoStatus);
+    UNREFERENCED_PARAMETER(CompressedDataInfo);
+    UNREFERENCED_PARAMETER(CompressedDataInfoLength);
+    UNREFERENCED_PARAMETER(DeviceObject);
 
     BOOLEAN bStatus = FALSE;
 
@@ -481,6 +584,16 @@ AFSFastIoWriteCompressed( IN struct _FILE_OBJECT *FileObject,
                           IN ULONG CompressedDataInfoLength,
                           IN struct _DEVICE_OBJECT *DeviceObject)
 {
+    UNREFERENCED_PARAMETER(FileObject);
+    UNREFERENCED_PARAMETER(FileOffset);
+    UNREFERENCED_PARAMETER(Length);
+    UNREFERENCED_PARAMETER(LockKey);
+    UNREFERENCED_PARAMETER(Buffer);
+    UNREFERENCED_PARAMETER(MdlChain);
+    UNREFERENCED_PARAMETER(IoStatus);
+    UNREFERENCED_PARAMETER(CompressedDataInfo);
+    UNREFERENCED_PARAMETER(CompressedDataInfoLength);
+    UNREFERENCED_PARAMETER(DeviceObject);
 
     BOOLEAN bStatus = FALSE;
 
@@ -492,6 +605,9 @@ AFSFastIoMdlReadCompleteCompressed( IN struct _FILE_OBJECT *FileObject,
                                     IN PMDL MdlChain,
                                     IN struct _DEVICE_OBJECT *DeviceObject)
 {
+    UNREFERENCED_PARAMETER(FileObject);
+    UNREFERENCED_PARAMETER(MdlChain);
+    UNREFERENCED_PARAMETER(DeviceObject);
 
     BOOLEAN bStatus = FALSE;
 
@@ -504,6 +620,10 @@ AFSFastIoMdlWriteCompleteCompressed( IN struct _FILE_OBJECT *FileObject,
                                      IN PMDL MdlChain,
                                      IN struct _DEVICE_OBJECT *DeviceObject)
 {
+    UNREFERENCED_PARAMETER(FileObject);
+    UNREFERENCED_PARAMETER(FileOffset);
+    UNREFERENCED_PARAMETER(MdlChain);
+    UNREFERENCED_PARAMETER(DeviceObject);
 
     BOOLEAN bStatus = FALSE;
 
@@ -515,6 +635,9 @@ AFSFastIoQueryOpen( IN struct _IRP *Irp,
                     OUT PFILE_NETWORK_OPEN_INFORMATION NetworkInformation,
                     IN struct _DEVICE_OBJECT *DeviceObject)
 {
+    UNREFERENCED_PARAMETER(Irp);
+    UNREFERENCED_PARAMETER(NetworkInformation);
+    UNREFERENCED_PARAMETER(DeviceObject);
 
     BOOLEAN bStatus = FALSE;
 
