@@ -265,7 +265,8 @@ AFSConfigureTrace( IN AFSTraceConfigCB *TraceInfo)
 
         if( TraceInfo->TraceLevel == AFSTraceLevel &&
             TraceInfo->TraceBufferLength == AFSDbgBufferLength &&
-            TraceInfo->Subsystem == AFSTraceComponent)
+            TraceInfo->Subsystem == AFSTraceComponent &&
+            TraceInfo->DebugFlags == AFSDebugFlags)
         {
 
             //
@@ -360,7 +361,16 @@ AFSConfigureTrace( IN AFSTraceConfigCB *TraceInfo)
                 DbgPrint("AFSConfigureTrace Failed to set debug buffer length in registry Status %08lX\n", ntStatus);
             }
 
-            AFSDbgBufferLength = TraceInfo->TraceBufferLength * 1024;
+	    if ( TraceInfo->TraceBufferLength > AFS_DBG_LOG_MAXLENGTH)
+	    {
+
+		AFSDbgBufferLength = AFS_DBG_LOG_MAXLENGTH * 1024;
+	    }
+	    else
+	    {
+
+		AFSDbgBufferLength = TraceInfo->TraceBufferLength * 1024;
+	    }
 
             ClearFlag( AFSDbgLogFlags, AFS_DBG_LOG_WRAPPED);
 
@@ -419,6 +429,31 @@ try_exit:
     return ntStatus;
 }
 
+NTSTATUS
+AFSGetTraceConfig( OUT AFSTraceConfigCB *TraceInfo)
+{
+    NTSTATUS ntStatus = STATUS_SUCCESS;
+
+    __Enter
+    {
+
+	AFSAcquireExcl( &AFSDbgLogLock,
+			TRUE);
+
+	TraceInfo->TraceLevel = AFSTraceLevel;
+
+	TraceInfo->TraceBufferLength = AFSDbgBufferLength;
+
+	TraceInfo->Subsystem = AFSTraceComponent;
+
+	TraceInfo->DebugFlags = AFSDebugFlags;
+
+	AFSReleaseResource( &AFSDbgLogLock);
+    }
+
+    return ntStatus;
+}
+
 NTSTATUS
 AFSGetTraceBuffer( IN ULONG TraceBufferLength,
                    OUT void *TraceBuffer,
