@@ -61,6 +61,8 @@ extern HANDLE              AFSSysProcess;
 
 extern UNICODE_STRING      AFSServerName;
 
+extern UNICODE_STRING      AFSMountRootName;
+
 extern AFSVolumeCB        *AFSGlobalRoot;
 
 extern UNICODE_STRING      AFSPIOCtlName;
@@ -77,6 +79,8 @@ extern PAFSProcessRequest  AFSProcessRequest;
 
 extern PAFSDbgLogMsg       AFSDbgLogMsg;
 
+extern PAFSDbgLogMsg       AFSDebugTraceFnc;
+
 extern PAFSAddConnectionEx AFSAddConnectionEx;
 
 extern PAFSExAllocatePoolWithTag   AFSExAllocatePoolWithTag;
@@ -99,6 +103,8 @@ extern PAFSRtlSetGroupSecurityDescriptor AFSRtlSetGroupSecurityDescriptor;
 
 extern SID_IDENTIFIER_AUTHORITY SeWorldSidAuthority;
 
+extern RTL_OSVERSIONINFOW AFSRtlSysVersion;
+
 }
 
 #endif /* _AFS_EXTERN_H */
