@@ -13,7 +13,6 @@
 extern "C" {
 #include <afsconfig.h>
 #include <afs/param.h>
-#include <afs/stds.h>
 #include <roken.h>
 }
 
@@ -76,15 +75,6 @@ BOOL CALLBACK Creds_DlgProc (HWND hDlg, UINT msg, WPARAM wp, LPARAM lp)
                break;
             }
          break;
-
-      case WM_HELP:
-         if (IsWindow (GetDlgItem (hDlg, IDC_CREDS_REMIND)))
-            WinHelp (hDlg, g.szHelpFile, HELP_CONTEXT, IDH_AFSCREDS_TAB_TOKENS);
-         else if (IsServiceRunning())
-            WinHelp (hDlg, g.szHelpFile, HELP_CONTEXT, IDH_AFSCREDS_TAB_NOTOKENS_RUNNING);
-         else // (!IsServiceRunning())
-            WinHelp (hDlg, g.szHelpFile, HELP_CONTEXT, IDH_AFSCREDS_TAB_NOTOKENS_STOPPED);
-         break;
       }
 
    return FALSE;
@@ -241,7 +231,7 @@ void ShowObtainCreds (BOOL fExpiring, LPTSTR pszCell)
         return;
     oc->parent = (IsWindowVisible (g.hMain)) ? g.hMain : NULL;
     oc->type = fExpiring ? IDD_NEWCREDS_EXPIRE : IDD_NEWCREDS;
-    oc->cell = _strdup(pszCell);
+    oc->cell = strdup(pszCell);
 
     HANDLE thread = 0;
     ULONG  threadID = 123;
@@ -293,10 +283,6 @@ BOOL CALLBACK NewCreds_DlgProc (HWND hDlg, UINT msg, WPARAM wp, LPARAM lp)
                break;
             }
          break;
-
-      case WM_HELP:
-         WinHelp (hDlg, g.szHelpFile, HELP_CONTEXT, IDH_AFSCREDS_NEWTOKENS);
-         break;
       }
    return FALSE;
 }
