@@ -22,7 +22,7 @@
  * SOFTWARE.
  */
 
-/* $Id: afsconfigdlg.c,v 1.1.2.4 2007/06/17 04:50:29 jaltman Exp $ */
+/* $Id$ */
 
 #include<afscred.h>
 #include<kherror.h>
@@ -234,7 +234,7 @@ set_service_status(HWND hwnd) {
     static DWORD wait_start = 0;
     DWORD status = 0;
     DWORD wait_hint = 0;
-    int i;
+    unsigned int i;
     wchar_t status_strings_csv[1024];
     wchar_t status_strings_ms[1024];
     khm_size cb;
@@ -457,7 +457,7 @@ afs_cfg_main_proc(HWND hwnd,
                                 L"ImagePath",
                                 NULL, NULL,
                                 (LPBYTE) imagepath,
-                                &cb);
+                                (DWORD *)&cb);
             if (l != ERROR_SUCCESS)
                 goto _close_key;
 
@@ -480,10 +480,10 @@ afs_cfg_main_proc(HWND hwnd,
                 goto _free_buffer;
 
             cb = 0;
-            if (!VerQueryValue(ver_info, 
+            if (!VerQueryValue(ver_info,
                                L"\\VarFileInfo\\Translation",
                                (LPVOID*) &translations,
-                               &cb) ||
+                               (PUINT)&cb) ||
                 cb == 0)
                 goto _free_buffer;
 
@@ -495,7 +495,7 @@ afs_cfg_main_proc(HWND hwnd,
             if (!VerQueryValue(ver_info,
                                blockname,
                                (LPVOID*) &value,
-                               &cb) ||
+                               (PUINT)&cb) ||
                 cb == 0)
                 goto _free_buffer;
 
@@ -509,7 +509,7 @@ afs_cfg_main_proc(HWND hwnd,
             if (!VerQueryValue(ver_info,
                                blockname,
                                (LPVOID*) &value,
-                               &cb) ||
+                               (PUINT)&cb) ||
                 cb == 0)
                 goto _free_buffer;
 
@@ -556,11 +556,11 @@ afs_cfg_main_proc(HWND hwnd,
             break;
 
         case MAKEWPARAM(IDC_CFG_CPL, BN_CLICKED):
-            if (32 >= (LRESULT) ShellExecute (NULL, NULL, 
+            if (32 >= (LRESULT) ShellExecute (NULL, NULL,
                                               L"AFS_CONFIG.EXE", NULL,
                                               NULL, SW_SHOW)) {
-                MessageBox(NULL, 
-                           L"Can't find file AFS_CONFIG.EXE", 
+                MessageBox(NULL,
+                           L"Can't find file AFS_CONFIG.EXE",
                            L"Error", MB_OK);
             }
             break;
