@@ -113,7 +113,10 @@ afs_mount(struct mount *mp, char *path, caddr_t data, struct nameidata *ndp, CTX
     memset(mnt_stat->f_mntfromname, 0, MNAMELEN);
 
     if (data == 0) {
-	strcpy(mnt_stat->f_mntfromname, "AFS");
+	if (strlcpy(mnt_stat->f_mntfromname, "AFS", MNAMELEN) >= MNAMELEN) {
+	    AFS_GUNLOCK();
+	    return ENAMETOOLONG;
+	}
 	/* null terminated string "AFS" will fit, just leave it be. */
 	vfs_setfsprivate(mp, NULL);
     } else {
@@ -125,7 +128,10 @@ afs_mount(struct mount *mp, char *path, caddr_t data, struct nameidata *ndp, CTX
 	memset(volName + size, 0, MNAMELEN - size);
 
 	if (volName[0] == 0) {
-	    strcpy(mnt_stat->f_mntfromname, "AFS");
+	    if (strlcpy(mnt_stat->f_mntfromname, "AFS", MNAMELEN) >= MNAMELEN) {
+		AFS_GUNLOCK();
+		return ENAMETOOLONG;
+	    }
 	    vfs_setfsprivate(mp, &afs_rootFid);
 	} else {
 	    struct cell *localcell = afs_GetPrimaryCell(READ_LOCK);
@@ -196,7 +202,7 @@ afs_unmount(struct mount *mp, int flags, CTX_TYPE ctx)
 	        vflush(mp, NULLVP, FORCECLOSE/*0*/);
 		AFS_GLOCK();
 		afs_globalVFS = 0;
-		afs_shutdown();
+		afs_shutdown(AFS_WARM);
 	    } else {
 		AFS_GUNLOCK();
 		return EBUSY;
@@ -281,12 +287,12 @@ again:
                     needref=1;
                 }
 	    } else
-		error = ENOENT;
+		error = EIO;
 	}
     }
     if (tvp) {
 #ifndef AFS_DARWIN80_ENV /* KPI callers don't need a usecount reference */
-	osi_vnhold(tvp, 0);
+	osi_Assert(osi_vnhold(tvp) == 0);
 	AFS_GUNLOCK();
 	vn_lock(AFSTOV(tvp), LK_EXCLUSIVE | LK_RETRY, p);
 	AFS_GLOCK();
@@ -357,16 +363,9 @@ afs_statfs(struct mount *mp, STATFS_TYPE *abp, CTX_TYPE ctx)
     abp->f_bsize = mp->vfs_bsize;
     abp->f_iosize = mp->vfs_bsize;
 #endif
-#if 0
-    abp->f_type = MOUNT_AFS;
-#endif
 
-    /* Fake a high number below to satisfy programs that use the statfs call
-     * to make sure that there's enough space in the device partition before
-     * storing something there.
-     */
     abp->f_blocks = abp->f_bfree = abp->f_bavail = abp->f_files =
-      abp->f_ffree = 0x7fffffff;
+      abp->f_ffree = AFS_VFS_FAKEFREE;
 
     if (abp != sysstat) {
         abp->f_fsid.val[0] = sysstat->f_fsid.val[0];
