@@ -26,6 +26,7 @@
 #include <afs/budb_client.h>
 #include <afs/bucoord_prototypes.h>
 #include <afs/butm_prototypes.h>
+#include <afs/afsint.h>
 #include <afs/volser.h>
 #include <afs/volser_prototypes.h>
 #include <afs/com_err.h>
@@ -50,7 +51,7 @@ extern int forcemultiple;
 #endif
 
 /* XBSA Global Parameters */
-afs_int32 xbsaType;
+extern afs_int32 xbsaType;
 #ifdef xbsa
 struct butx_transactionInfo butxInfo;
 #endif
@@ -981,15 +982,12 @@ xbsaRestoreVolumeData(struct rx_call *call, void *rock)
     afs_int32 headBytes, tailBytes, w;
     afs_int32 taskId;
     struct volumeHeader volTrailer;
-    afs_int32 vtsize = 0;
     int found;
     struct dumpNode *nodePtr;
-    struct tc_restoreDesc *Restore;
     afs_int32 bytesRead, tbuffersize, endData = 0;
     char *buffer = (char *)bufferBlock, tbuffer[256];
 
     nodePtr = rparamsPtr->nodePtr;
-    Restore = nodePtr->restores;
     taskId = nodePtr->taskID;
 
     /* Read the volume fragment one block at a time until
@@ -1638,7 +1636,7 @@ Restorer(void *param) {
     time_t startTime, endTime;
     afs_int32 goodrestore = 0;
 
-    afs_pthread_setname_self("restorer");
+    opr_threadname_set("restorer");
     taskId = newNode->taskID;
     setStatus(taskId, DRIVE_WAIT);
     EnterDeviceQueue(deviceLatch);
@@ -2104,7 +2102,7 @@ Labeller(void *param)
     afs_uint32 taskId;
     afs_int32 code = 0;
 
-    afs_pthread_setname_self("labeller");
+    opr_threadname_set("labeller");
     taskId = labelIfPtr->taskId;
     setStatus(taskId, DRIVE_WAIT);
     EnterDeviceQueue(deviceLatch);
