@@ -91,6 +91,18 @@
 #define SYS_NAME_ID_x86_darwin_180       538
 #define SYS_NAME_ID_amd64_darwin_180     539
 #define SYS_NAME_ID_arm_darwin_180       540
+#define SYS_NAME_ID_x86_darwin_190       541
+#define SYS_NAME_ID_amd64_darwin_190     542
+#define SYS_NAME_ID_arm_darwin_190       543
+#define SYS_NAME_ID_x86_darwin_200       544
+#define SYS_NAME_ID_amd64_darwin_200     545
+#define SYS_NAME_ID_arm_darwin_200       546
+#define SYS_NAME_ID_amd64_darwin_210     547
+#define SYS_NAME_ID_arm_darwin_210       548
+#define SYS_NAME_ID_amd64_darwin_220     549
+#define SYS_NAME_ID_arm_darwin_220       550
+#define SYS_NAME_ID_amd64_darwin_230     551
+#define SYS_NAME_ID_arm_darwin_230       552
 
 #define SYS_NAME_ID_next_mach20		 601
 #define SYS_NAME_ID_next_mach30		 602
@@ -103,6 +115,9 @@
 #define SYS_NAME_ID_rs_aix52		 706
 #define SYS_NAME_ID_rs_aix53		 707
 #define SYS_NAME_ID_rs_aix61		 708
+#define SYS_NAME_ID_rs_aix72		 709
+#define SYS_NAME_ID_rs_aix71		 710
+#define SYS_NAME_ID_rs_aix73		 711
 
 #define SYS_NAME_ID_sun3_411		 906
 #define SYS_NAME_ID_sun3x_411		 912
@@ -237,7 +252,12 @@
 #define SYS_NAME_ID_i386_fbsd_104      2154
 #define SYS_NAME_ID_i386_fbsd_110      2140
 #define SYS_NAME_ID_i386_fbsd_111      2141
+#define SYS_NAME_ID_i386_fbsd_112      2142
+#define SYS_NAME_ID_i386_fbsd_113      2143
 #define SYS_NAME_ID_i386_fbsd_120      2160
+#define SYS_NAME_ID_i386_fbsd_121      2161
+#define SYS_NAME_ID_i386_fbsd_122      2162
+#define SYS_NAME_ID_i386_fbsd_123      2163
 
 #define SYS_NAME_ID_ia64_linux2		2200
 #define SYS_NAME_ID_ia64_linux22	2201
@@ -350,7 +370,12 @@
 #define SYS_NAME_ID_amd64_fbsd_104     3054
 #define SYS_NAME_ID_amd64_fbsd_110     3040
 #define SYS_NAME_ID_amd64_fbsd_111     3041
+#define SYS_NAME_ID_amd64_fbsd_112     3042
+#define SYS_NAME_ID_amd64_fbsd_113     3043
 #define SYS_NAME_ID_amd64_fbsd_120     3060
+#define SYS_NAME_ID_amd64_fbsd_121     3061
+#define SYS_NAME_ID_amd64_fbsd_122     3062
+#define SYS_NAME_ID_amd64_fbsd_123     3063
 
 #define SYS_NAME_ID_amd64_w2k           3400
 
@@ -391,6 +416,8 @@
 #define SYS_NAME_ID_arm64_linux2	4100
 #define SYS_NAME_ID_arm64_linux26	4103
 
+#define SYS_NAME_ID_ppc64le_linux26	4203
+
 #define	AFS_REALM_SZ	64
 
 /* Specifies the number of equivalent local realm names */
